#pragma once

#include "stdafx.h"

using namespace std;

BOOL SD_OnInitDialog(HWND hDlg, HWND /*hwndFocus*/, LPARAM /*lParam*/)
{

	RECT rc = {};
	GetClientRect(hDlg, &rc);

	const SIZE sz = { rc.right - rc.left, rc.bottom - rc.top };

	SCROLLINFO si = {};
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_PAGE | SIF_POS | SIF_RANGE;
	si.nPos = si.nMin = 1;

	si.nMax = sz.cx;
	si.nPage = sz.cx;
	SetScrollInfo(hDlg, SB_HORZ, &si, FALSE);

	si.nMax = sz.cy;
	si.nPage = sz.cy;
	SetScrollInfo(hDlg, SB_VERT, &si, FALSE);

	return FALSE;
}

void SD_OnCommand(HWND hDlg, int id, HWND /*hwndCtl*/, UINT /*codeNotify*/)
{
    switch(id)
    {
    case IDOK:
    case IDCANCEL:
        EndDialog(hDlg, 0);
        break;
    }
}

void SD_OnSize(HWND hDlg, UINT state, int cx, int cy)
{
    if(state != SIZE_RESTORED && state != SIZE_MAXIMIZED)
        return;

    SCROLLINFO si = {};
    si.cbSize = sizeof(SCROLLINFO);

    const int bar[] = { SB_HORZ, SB_VERT };
    const int page[] = { cx, cy };

    for(size_t i = 0; i < ARRAYSIZE(bar); ++i)
    {
        si.fMask = SIF_PAGE;
        si.nPage = page[i];
        SetScrollInfo(hDlg, bar[i], &si, TRUE);

        si.fMask = SIF_RANGE | SIF_POS;
        GetScrollInfo(hDlg, bar[i], &si);

        const int maxScrollPos = si.nMax - (page[i] - 1);

         //Scroll client only if scroll bar is visible and window's
         //content is fully scrolled toward right and/or bottom side.
         //Also, update window's content on maximize.
        const bool needToScroll =
            (si.nPos != si.nMin && si.nPos == maxScrollPos) ||
            (state == SIZE_MAXIMIZED);

        if(needToScroll)
        {
            SD_ScrollClient(hDlg, bar[i], si.nPos);
        }
    }
}

void SD_OnHScroll(HWND hDlg, HWND /*hwndCtl*/, UINT code, int /*pos*/)
{
    SD_OnHVScroll(hDlg, SB_HORZ, code);
}

void SD_OnVScroll(HWND hDlg, HWND /*hwndCtl*/, UINT code, int /*pos*/)
{
    SD_OnHVScroll(hDlg, SB_VERT, code);
}

void SD_OnHVScroll(HWND hDlg, int bar, UINT code)
{
    const int scrollPos = SD_GetScrollPos(hDlg, bar, code);

    if(scrollPos == -1)
        return;

    SetScrollPos(hDlg, bar, scrollPos, TRUE);
    SD_ScrollClient(hDlg, bar, scrollPos);
}

void SD_ScrollClient(HWND hDlg, int bar, int pos)
{
    static int s_prevx = 1;
    static int s_prevy = 1;

    int cx = 0;
    int cy = 0;

    int& delta = (bar == SB_HORZ ? cx : cy);
    int& prev = (bar == SB_HORZ ? s_prevx : s_prevy);

    delta = prev - pos;
    prev = pos;

    if(cx || cy)
    {
        ScrollWindow(hDlg, cx, cy, NULL, NULL);
    }
}

int SD_GetScrollPos(HWND hDlg, int bar, UINT code)
{
    SCROLLINFO si = {};
    si.cbSize = sizeof(SCROLLINFO);
    si.fMask = SIF_PAGE | SIF_POS | SIF_RANGE | SIF_TRACKPOS;
    GetScrollInfo(hDlg, bar, &si);

    const int minPos = si.nMin;
    const int maxPos = si.nMax - (si.nPage - 1);

    int result = -1;

    switch(code)
    {
    case SB_LINEUP /*SB_LINELEFT*/:
        result = max(si.nPos - 1, minPos);
        break;

    case SB_LINEDOWN /*SB_LINERIGHT*/:
        result = min(si.nPos + 1, maxPos);
        break;

    case SB_PAGEUP /*SB_PAGELEFT*/:
        result = max(si.nPos - (int)si.nPage, minPos);
        break;

    case SB_PAGEDOWN /*SB_PAGERIGHT*/:
        result = min(si.nPos + (int)si.nPage, maxPos);
        break;

    case SB_THUMBPOSITION:
        // do nothing
        break;

    case SB_THUMBTRACK:
        result = si.nTrackPos;
        break;

    case SB_TOP /*SB_LEFT*/:
        result = minPos;
        break;

    case SB_BOTTOM /*SB_RIGHT*/:
        result = maxPos;
        break;

    case SB_ENDSCROLL:
        // do nothing
        break;
    }

    return result;
}